//------------------------------------------------------------------------------
//
// This software is provided "AS IS".  360Commerce MAKES NO
// REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,
// EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NON-INFRINGEMENT. 360Commerce shall not be liable for
// any damages suffered as a result of using, modifying or distributing this
// software or its derivatives. Permission to use, copy, modify, and distribute
// the software and its documentation for any purpose is hereby granted.
//
// BumpBarPanel.java - The BumpBar panel of POStest
//
//------------------------------------------------------------------------------
package com.jpos.POStest;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import jpos.*;
import jpos.events.*;


public class BumpBarPanel extends Component implements StatusUpdateListener, ActionListener {

    protected MainButtonPanel mainButtonPanel;
    
    private BumpBar bumpBar;

    private String defaultLogicalName = "defaultBumpBar";


    private JCheckBox deviceEnabledCB;
    private JCheckBox freezeEventsCB;
    
    private JTextArea statusTextArea;

    public BumpBarPanel() 
    {
          bumpBar = new BumpBar();
    }

    public Component make() {
        
        JPanel mainPanel = new JPanel(false);
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

//        MethodListener methodListener = new MethodListener();

        mainButtonPanel = new MainButtonPanel(this,defaultLogicalName);
        mainPanel.add(mainButtonPanel);

        JPanel buttonPanel = new JPanel();

        buttonPanel.setMaximumSize(new Dimension(Short.MAX_VALUE, 30)); 

        mainPanel.add(buttonPanel);


        JPanel subPanel = new JPanel();
        subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.X_AXIS));

        JPanel propPanel = new JPanel();
        propPanel.setLayout(new BoxLayout(propPanel, BoxLayout.Y_AXIS));


        deviceEnabledCB = new JCheckBox("Device enabled");
        propPanel.add(deviceEnabledCB);
        freezeEventsCB = new JCheckBox("Freeze events");
        propPanel.add(freezeEventsCB);
        propPanel.add(Box.createVerticalGlue());
        subPanel.add(propPanel);
       
        deviceEnabledCB.setEnabled(false);
        freezeEventsCB.setEnabled(false);

        CheckBoxListener cbListener = new CheckBoxListener();
        deviceEnabledCB.addItemListener(cbListener);
        freezeEventsCB.addItemListener(cbListener);



        JPanel statusPanel = new JPanel();
        statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.Y_AXIS));
        JLabel label = new JLabel("Not yet implemented.");
        label.setAlignmentX(Component.CENTER_ALIGNMENT);
        statusPanel.add(label);
        statusPanel.add(Box.createHorizontalGlue());

        subPanel.add(statusPanel);

        mainPanel.add(subPanel);
        mainPanel.add(Box.createVerticalGlue());

        return mainPanel;
    }

    public void statusUpdateOccurred(StatusUpdateEvent sue)
    {
        System.out.println("BumpBar received status update event.");
    }


   /** Listens to the method buttons. */

        public void actionPerformed(ActionEvent ae) {
             mainButtonPanel.action(ae);
             String logicalName = mainButtonPanel.getLogicalName();
             if(ae.getActionCommand().equals("open")){
                 try{
                     if(logicalName.equals("")){
                         logicalName = defaultLogicalName;
                     }
                     bumpBar.addStatusUpdateListener(this);

                     bumpBar.open(logicalName);
                     deviceEnabledCB.setEnabled(false);
                     freezeEventsCB.setEnabled(true);
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to open \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception " + e);
                 }
             }
             else if(ae.getActionCommand().equals("claim")){
                 try{
                     bumpBar.claim(0);

                     deviceEnabledCB.setEnabled(true);
                     freezeEventsCB.setEnabled(true);
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to claim \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception " + e);
                 }
             }
             else if(ae.getActionCommand().equals("release")){
                 try{
                     bumpBar.release();

                     deviceEnabledCB.setEnabled(false);
                     freezeEventsCB.setEnabled(true);

                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to release \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception " + e);
                 }
             }
             else if(ae.getActionCommand().equals("close")){
                 try{
                     bumpBar.close();
                     deviceEnabledCB.setEnabled(false);
                     freezeEventsCB.setEnabled(false);

                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to close \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception " + e);
                 }
             }
             else if(ae.getActionCommand().equals("info")){
             	try{
             		String ver = new Integer(bumpBar.getDeviceServiceVersion()).toString();
             		String msg = "Service Description: " + bumpBar.getDeviceServiceDescription();
             		msg = msg + "\nService Version: v"+ new Integer(ver.substring(0,1)) + "." + new Integer(ver.substring(1,4)) + "." + new Integer(ver.substring(4,7));
             		ver = new Integer(bumpBar.getDeviceControlVersion()).toString();
             		msg += "\n\nControl Description: " + bumpBar.getDeviceControlDescription();
             		msg += "\nControl Version: v"+ new Integer(ver.substring(0,1)) + "." + new Integer(ver.substring(1,4)) + "." + new Integer(ver.substring(4,7));
             		msg += "\n\nPhysical Device Name: " + bumpBar.getPhysicalDeviceName();
             		msg += "\nPhysical Device Description: " + bumpBar.getPhysicalDeviceDescription();
             		
             		msg += "\n\nProperties:\n------------------------";
             		
             		msg += "\nCapPowerReporting: " + (bumpBar.getCapPowerReporting() == JposConst.JPOS_PR_ADVANCED ? "Advanced" : (bumpBar.getCapPowerReporting() == JposConst.JPOS_PR_STANDARD ? "Standard" : "None"));
             		JOptionPane.showMessageDialog(null, msg, "Info", JOptionPane.INFORMATION_MESSAGE);

             	}catch(JposException e){
             		JOptionPane.showMessageDialog(null, "Exception in Info\nException: "+ e.getMessage(), "Exception", JOptionPane.ERROR_MESSAGE);
             		System.err.println("Jpos exception " + e);
             	}
             }
            try
            {
                deviceEnabledCB.setSelected(bumpBar.getDeviceEnabled());
                freezeEventsCB.setSelected(bumpBar.getFreezeEvents());
            }
            catch(JposException je)
            {
                System.err.println("BumpBarPanel: MethodListener: JposException");
            }
        }


    class CheckBoxListener implements ItemListener 
    {
        public void itemStateChanged(ItemEvent e) 
        {
            Object source = e.getItemSelectable();
            try
            {
                if (source == deviceEnabledCB){
                    if (e.getStateChange() == ItemEvent.DESELECTED){
                        bumpBar.setDeviceEnabled(false);
                    }else{
                        bumpBar.setDeviceEnabled(true);
                    }
                }else if (source == freezeEventsCB){
                    if (e.getStateChange() == ItemEvent.DESELECTED){
                        bumpBar.setFreezeEvents(false);
                    }else{
                        bumpBar.setFreezeEvents(true);
                    }
                }
            }
            catch(JposException je)
            {
                System.err.println("BumpBarPanel: CheckBoxListener: Jpos Exception" + e);
            }
         }
     }    
}
